# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ian Weller <ianweller@fedoraproject.org>
#           Ralph Bean <rbean@redhat.com>
#
""" Tests for git messages """

import os
import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


here = os.path.dirname(os.path.abspath(__file__))
with open(here + '/example.patch', 'r') as f:
    full_patch = f.read()


class TestGitReceiveLegacyModified(Base):
    """ Sample message from the first generation of git-category messages that
    have been modified in datanommer to match the new topics.
    """

    expected_title = "git.receive"
    expected_subti = ('rbean@redhat.com pushed to datanommer (master).  "Try '
                      'removing requirement on python-bunch."')
    expected_secondary_icon = (
        'https://seccdn.libravatar.org/avatar/'
        '1a0d2acfddb1911ecf55da42cfa34710'
        '?s=64&d=retro')
    expected_packages = set(['datanommer'])
    expected_usernames = set()
    expected_objects = set(['datanommer/datanommer.spec'])
    expected_link = ("http://pkgs.fedoraproject.org/cgit/datanommer.git/commit"
                     "/?h=master&id=66abdea4014eb2f0745fc38f86e20c7d7009237e")

    msg = {
        "i": 1,
        "msg": {
            "commit": {
                "branch": "master",
                "email": "rbean@redhat.com",
                "message": "Try removing requirement on python-bunch.\n",
                "name": "Ralph Bean",
                "repo": "datanommer",
                "rev": "66abdea4014eb2f0745fc38f86e20c7d7009237e",
                "stats": {
                    "files": {
                        "datanommer.spec": {
                            "deletions": 6,
                            "insertions": 4,
                            "lines": 10
                        }
                    },
                    "total": {
                        "deletions": 6,
                        "files": 1,
                        "insertions": 4,
                        "lines": 10
                    }
                },
                "summary": "Try removing requirement on python-bunch."
            }
        },
        "timestamp": 1349735155.0,
        "topic": "org.fedoraproject.prod.git.receive"
    }

class TestSCMSuperLegacy(Base):
    """ Support super-duper oldschool git messages.  :(:( """

    expected_title = "git.receive.valgrind.master"
    expected_subti = 'mjw pushed to valgrind (master).  ' + \
        '"Clear CFLAGS CXXFLAGS LDFLAGS. (..more)"'
    expected_link = "http://pkgs.fedoraproject.org/cgit/" + \
        "valgrind.git/commit/" + \
        "?h=master&id=7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1"
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = ("https://seccdn.libravatar.org/avatar/"
        "0f32874b1ae3083205c874c83cd2d21715c89b8645483f353e90ae499c67c944"
        "?s=64&d=retro")
    expected_usernames = set(['mjw'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/valgrind.spec'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.receive.valgrind.master",
        "msg": {
            "commit": {
                "stats": {
                    "files": {
                        "valgrind.spec": {
                            "deletions": 2,
                            "lines": 3,
                            "insertions": 1
                        }
                    },
                    "total": {
                        "deletions": 2,
                        "files": 1,
                        "insertions": 1,
                        "lines": 3
                    }
                },
                "name": "Mark Wielaard",
                "rev": "7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1",
                "summary": "Clear CFLAGS CXXFLAGS LDFLAGS.",
                "message": """Clear CFLAGS CXXFLAGS LDFLAGS.
                This is a bit of a hammer.""",
                "email": "mjw@redhat.com",
                "username": "mjw",
                "branch": "master",
            }
        }
    }


class TestSCMLegacy(Base):
    """ Support oldschool "fedpkg push" messages.  :( """

    expected_title = "git.receive.valgrind.master"
    expected_subti = 'mjw pushed to valgrind (master).  ' + \
        '"Clear CFLAGS CXXFLAGS LDFLAGS. (..more)"'
    expected_link = "http://pkgs.fedoraproject.org/cgit/" + \
        "valgrind.git/commit/" + \
        "?h=master&id=7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1"
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = ("https://seccdn.libravatar.org/avatar/"
        "0f32874b1ae3083205c874c83cd2d21715c89b8645483f353e90ae499c67c944"
        "?s=64&d=retro")
    expected_usernames = set(['mjw'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/valgrind.spec'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.receive.valgrind.master",
        "msg": {
            "commit": {
                "stats": {
                    "files": {
                        "valgrind.spec": {
                            "deletions": 2,
                            "lines": 3,
                            "insertions": 1
                        }
                    },
                    "total": {
                        "deletions": 2,
                        "files": 1,
                        "insertions": 1,
                        "lines": 3
                    }
                },
                "name": "Mark Wielaard",
                "rev": "7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1",
                "summary": "Clear CFLAGS CXXFLAGS LDFLAGS.",
                "message": """Clear CFLAGS CXXFLAGS LDFLAGS.
                This is a bit of a hammer.""",
                "email": "mjw@redhat.com",
                "username": "mjw",
                "branch": "master",
                "repo": "valgrind",
            }
        }
    }


class TestSCM(Base):
    """ Messages like this one are published when somebody runs "fedpkg push"
    on a package.  Sometimes, the git message may be multiple lines long like:
    """
    expected_title = "git.receive"
    expected_subti = 'mjw pushed to valgrind (master).  ' + \
        '"Clear CFLAGS CXXFLAGS LDFLAGS. (..more)"'
    expected_link = "http://pkgs.fedoraproject.org/cgit/" + \
        "valgrind.git/commit/" + \
        "?h=master&id=7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1"
    expected_long_form = "This commit already existed in another branch."
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = ("https://seccdn.libravatar.org/avatar/"
        "0f32874b1ae3083205c874c83cd2d21715c89b8645483f353e90ae499c67c944"
        "?s=64&d=retro")
    expected_usernames = set(['mjw'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/valgrind.spec'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.receive",
        "msg": {
            "commit": {
                "seen": True,
                "stats": {
                    "files": {
                        "valgrind.spec": {
                            "deletions": 2,
                            "lines": 3,
                            "insertions": 1
                        }
                    },
                    "total": {
                        "deletions": 2,
                        "files": 1,
                        "insertions": 1,
                        "lines": 3
                    }
                },
                "name": "Mark Wielaard",
                "rev": "7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1",
                "summary": "Clear CFLAGS CXXFLAGS LDFLAGS.",
                "message": """Clear CFLAGS CXXFLAGS LDFLAGS.
                This is a bit of a hammer.""",
                "email": "mjw@redhat.com",
                "username": "mjw",
                "branch": "master",
                "repo": "valgrind",
            }
        }
    }


class TestSCMSingleLine(Base):
    """ Messages like this one are published when somebody runs "fedpkg push"
    on a package.  The whole git message is included for each commit.
    """
    expected_title = "git.receive"
    expected_subti = 'spot pushed to ember (master).  ' + \
        '"another missing patch? ridiculous."'
    expected_link = "http://pkgs.fedoraproject.org/cgit/" + \
        "ember.git/commit/" + \
        "?h=master&id=aa2df80f3d8dd217c7cbfe2d3451190028f3fe14"
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = ("https://seccdn.libravatar.org/avatar/"
        "1e232310ef80ec8b34b0bc216864efd0d837f419e6988997cda8e98a28be48dd"
        "?s=64&d=retro")
    expected_usernames = set(['spot'])
    expected_packages = set(['ember'])
    expected_objects = set(['ember/ember-0.6.3-gcc47.patch'])

    msg = {
        "i": 1,
        "timestamp": 1352998154.368305,
        "topic": "org.fedoraproject.prod.git.receive",
        "msg": {
            "commit": {
                "username": "spot",
                "stats": {
                    "files": {
                        "ember-0.6.3-gcc47.patch": {
                            "deletions": 0,
                            "lines": 26,
                            "insertions": 26
                        }
                    },
                    "total": {
                        "deletions": 0,
                        "files": 1,
                        "insertions": 26,
                        "lines": 26
                    }
                },
                "name": "Tom Callaway",
                "rev": "aa2df80f3d8dd217c7cbfe2d3451190028f3fe14",
                "summary": "another missing patch? ridiculous.",
                "branch": "master",
                "repo": "ember",
                "message": "another missing patch? ridiculous.\n",
                "email": "spot@fedoraproject.org"
            }
        }
    }

if not 'FEDMSG_META_NO_NETWORK' in os.environ:
    TestGitReceiveLegacyModified.expected_long_form = full_patch


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Pierre-Yves Chibon <pingou@pingoured.fr>
#

from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url, email2fas
from fedmsg_meta_fedora_infrastructure import BaseProcessor

from fedmsg_meta_fedora_infrastructure.conglomerators.pagure import \
        pagure as pagure_conglomerator

import fedmsg.meta.base

def _get_project(msg, key='project'):
    ''' Return the project as `foo` or `user/foo` if the project is a
    fork.
    '''
    project = msg[key]['name']
    if msg[key]['parent']:
        user = msg[key]['user']['name']
        project = '/'.join([user, project])
    return project


def _git_receive_v1(msg, tmpl, **config):
    ''' Return the subtitle for the first version of pagure git.receive
    messages.
    '''
    repo = _get_project(msg['msg']['commit'], key='repo')
    email = msg['msg']['commit']['email']
    user = email2fas(email, **config)
    summ = msg['msg']['commit']['summary']
    whole = msg['msg']['commit']['message']
    if summ.strip() != whole.strip():
        summ += " (..more)"

    branch = msg['msg']['commit']['branch']
    if 'refs/heads/' in branch:
        branch = branch.replace('refs/heads/', '')
    return tmpl.format(user=user or email, repo=repo,
                       branch=branch, summary=summ)


def _git_receive_v2(msg, tmpl):
    ''' Return the subtitle for the second version of pagure git.receive
    messages.
    '''
    repo = _get_project(msg['msg'], key='repo')
    user = msg['msg']['agent']
    n_commits = msg['msg']['total_commits']
    branch = msg['msg']['branch']
    if 'refs/heads/' in branch:
        branch = branch.replace('refs/heads/', '')
    return tmpl.format(user=user, repo=repo,
                       branch=branch, n_commits=n_commits)


class PagureProcessor(BaseProcessor):
    topic_prefix_re = 'io\\.pagure\\.(dev|stg|prod)'

    __name__ = "pagure"
    __description__ = "Pagure forge"
    __link__ = "https://pagure.io"
    __docs__ = "https://pagure.io/pagure"
    __obj__ = "Pagure forge"
    __icon__ = ("https://apps.fedoraproject.org/packages/"
                "images/icons/package_128x128.png")

    conglomerators = [
        pagure_conglomerator.ByPR,
        pagure_conglomerator.ByIssue,
        pagure_conglomerator.ByNewStyleCommit,
        pagure_conglomerator.ByOldStyleCommit,
    ]


    def link(self, msg, **config):
        try:
            project = _get_project(msg['msg'])
        except KeyError:
            try:
                project = _get_project(msg['msg']['pullrequest'])
            except KeyError:
                project = "(unknown)"

        base_url = "https://pagure.io"

        tmpl = '{base_url}/{project}'
        if '/' in project:
            tmpl = '{base_url}/fork/{project}'
        if 'pagure.issue' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            if 'comment' in msg['topic']:
                if 'comment' in msg['msg']:
                    # This is for an edited issue..
                    return base_url
                comments = msg['msg']['issue']['comments']
                if comments:
                    tmpl += '/issue/{id}#comment-{comment}'
                    return tmpl.format(
                        base_url=base_url, project=project, id=issueid,
                        comment=comments[-1]['id'])
                else:
                    tmpl += '/issue/{id}'
                    return tmpl.format(
                        base_url=base_url, project=project, id=issueid)
            else:
                tmpl += '/issue/{id}'
                return tmpl.format(
                    base_url=base_url, project=project, id=issueid)
        elif 'pagure.pull-request' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            if 'comment' in msg['topic']:
                comments = msg['msg']['pullrequest']['comments']
                if comments:
                    tmpl += '/pull-request/{id}#comment-{comment}'
                    return tmpl.format(
                        base_url=base_url, project=project, id=prid,
                        comment=comments[-1]['id'])
                elif msg['msg'].get('comment'):
                    comment_id = msg['msg']['comment']['id']
                    tmpl += '/pull-request/{id}#comment-{comment}'
                    return tmpl.format(
                        base_url=base_url, project=project, id=prid,
                        comment=comment_id)
                else:
                    tmpl += '/pull-request/{id}'
                    return tmpl.format(
                        base_url=base_url, project=project, id=prid)
            else:
                tmpl += '/pull-request/{id}'
                return tmpl.format(
                    base_url=base_url, project=project, id=prid)
        elif 'pagure.project' in msg['topic']:
            return tmpl.format(base_url=base_url, project=project)
        elif 'pagure.git.receive' in msg['topic']:
            if 'commit' in msg['msg']:
                project = _get_project(msg['msg']['commit'], key='repo')
                item = msg['msg']['commit']['rev']
                tmpl += '/{item}'
            else:
                project = _get_project(msg['msg'], key='repo')
                item = msg['msg']['branch']
                if 'refs/heads/' in item:
                    item = item.replace('refs/heads/', '')
                tmpl += '/branch/{item}'
            return tmpl.format(
                base_url=base_url, project=project, item=item)

        else:
            return base_url

        return None

    def subtitle(self, msg, **config):
        try:
            project = _get_project(msg['msg'])
        except KeyError:
            try:
                project = _get_project(msg['msg']['pullrequest'])
            except KeyError:
                project = "(unknown)"
        user = msg['msg'].get('agent')

        if 'pagure.project.new' in msg['topic']:
            tmpl = self._(
                '{user} created a new project "{project}"'
            )
            return tmpl.format(user=user, project=project)
        elif 'pagure.issue.new' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            title = msg['msg']['issue']['title']
            tmpl = self._(
                '{user} opened a new ticket {project}#{id}: "{title}"')
            return tmpl.format(
                user=user, project=project, title=title, id=issueid)
        elif 'pagure.issue.drop' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            title = msg['msg']['issue']['title']
            tmpl = self._(
                '{user} deleted ticket {project}#{id}: "{title}"')
            return tmpl.format(
                user=user,project=project,title=title,id=issueid)
        elif 'pagure.issue.comment.edited' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            title = msg['msg']['issue']['title']
            tmpl = self._(
                '{user} edited a comment on ticket {project}#{id}: "{title}"')
            return tmpl.format(
                user=user, project=project, title=title, id=issueid)
        elif 'pagure.issue.comment.added' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            title = msg['msg']['issue']['title']
            tmpl = self._(
                '{user} commented on ticket {project}#{id}: "{title}"')
            return tmpl.format(
                user=user, project=project, title=title, id=issueid)
        elif 'pagure.issue.tag.added' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            tags = msg['msg']['tags']
            tags = fedmsg.meta.base.BaseConglomerator.list_to_series(tags)
            tmpl = self._(
                '{user} tagged ticket {project}#{id}: {tags}')
            return tmpl.format(
                user=user, project=project, id=issueid, tags=tags)
        elif 'pagure.issue.tag.removed' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            tags = msg['msg']['tags']
            tags = fedmsg.meta.base.BaseConglomerator.list_to_series(tags)
            tmpl = self._(
                '{user} removed the {tags} tags from ticket {project}#{id}')
            return tmpl.format(
                user=user, project=project, id=issueid, tags=tags)
        elif 'pagure.issue.assigned.added' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            assignee = msg['msg']['issue']['assignee']['name']
            tmpl = self._(
                '{user} assigned ticket {project}#{id} to {assignee}')
            return tmpl.format(
                user=user, project=project, id=issueid, assignee=assignee)
        elif 'pagure.issue.assigned.reset' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            tmpl = self._(
                '{user} reset the assignee of ticket {project}#{id}')
            return tmpl.format(user=user, project=project, id=issueid)
        elif 'pagure.issue.dependency.added' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            dep_id = msg['msg']['added_dependency']
            tmpl = self._(
                '{user} added ticket {project}#{id} as a dependency of '
                'ticket {project}#{dep_id}'
            )
            return tmpl.format(
                user=user, project=project, id=issueid,
                dep_id=dep_id)
        elif 'pagure.issue.dependency.removed' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            removed = msg['msg']['removed_dependency']
            tmpl = self._(
                '{user} removed ticket {project}#{id} as a dependency '
                'of ticket {project}#{removed}'
            )
            return tmpl.format(
                user=user, project=project, id=issueid, removed=removed)
        elif 'pagure.issue.edit' in msg['topic']:
            issueid = msg['msg']['issue']['id']
            fields = msg['msg']['fields']
            if fields == ['status']:
                status = msg['msg']['issue']['status']
                tmpl = self._(
                    '{user} set the status of ticket {project}#{id} to: '
                    '{status}')
                return tmpl.format(
                    user=user, project=project, id=issueid, status=status)
            fields = fedmsg.meta.base.BaseConglomerator.list_to_series(fields)
            tmpl = self._(
                '{user} edited the {fields} fields of ticket '
                '{project}#{id}'
            )
            return tmpl.format(
                user=user, project=project, id=issueid, fields=fields)
        elif 'pagure.project.edit' in msg['topic']:
            fields = msg['msg']['fields']
            fields = fedmsg.meta.base.BaseConglomerator.list_to_series(fields)
            tmpl = self._(
                '{user} edited the {fields} fields of project {project}')
            return tmpl.format(user=user, project=project, fields=fields)
        elif 'pagure.project.user.added' in msg['topic']:
            new_user = msg['msg']['new_user']
            tmpl = self._(
                '{user} added "{new_user}" to project {project}'
            )
            return tmpl.format(user=user, project=project, new_user=new_user)
        elif 'pagure.project.tag.removed' in msg['topic']:
            tags = msg['msg']['tags']
            tags = fedmsg.meta.base.BaseConglomerator.list_to_series(tags)
            tmpl = self._(
                '{user} removed tags "{tags}" from project {project}'
            )
            return tmpl.format(user=user, project=project, tags=tags)
        elif 'pagure.project.tag.edited' in msg['topic']:
            old_tag = msg['msg']['old_tag']
            new_tag = msg['msg']['new_tag']
            tmpl = self._(
                '{user} altered tags on project {project} from '
                '"{old_tag}" to "{new_tag}"'
            )
            return tmpl.format(
                user=user, project=project,
                old_tag=old_tag, new_tag=new_tag)
        elif 'pagure.project.forked' in msg['topic']:
            old_project = msg['msg']['project']['parent']['name']
            tmpl = self._(
                '{user} forked project "{old_project}" to "{project}"'
            )
            return tmpl.format(
                user=user, old_project=old_project, project=project)
        elif 'pagure.pull-request.comment.added' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            tmpl = self._(
                '{user} commented on pull-request#{id} of project '
                '"{project}"'
            )
            return tmpl.format(user=user, id=prid, project=project)
        elif 'pagure.pull-request.comment.edited' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            tmpl = self._(
                '{user} edited a comment on pull-request#{id} of project '
                '"{project}"'
            )
            return tmpl.format(user=user, id=prid, project=project)
        elif 'pagure.pull-request.closed' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            merged = msg['msg']['merged']
            if merged:
                tmpl = self._(
                    '{user} merged pull-request#{id} of project '
                    '"{project}"'
                )
            else:
                tmpl = self._(
                    '{user} closed (without merging) pull-request#{id} '
                    'of project "{project}"'
                )
            return tmpl.format(user=user, id=prid, project=project)
        elif 'pagure.pull-request.new' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            title = msg['msg']['pullrequest']['title']
            tmpl = self._(
                '{user} opened pull-request#{id}: "{title}" on '
                'project "{project}"'
            )
            return tmpl.format(
                user=user, id=prid, project=project, title=title)
        elif 'pagure.pull-request.flag.added' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            username = msg['msg']['flag']['username']
            comment = msg['msg']['flag']['comment']
            tmpl = self._(
                '{username} flagged {project}#{id} with "{comment}"'
            )
            return tmpl.format(
                username=username, id=prid, comment=comment, project=project)
        elif 'pagure.pull-request.flag.updated' in msg['topic']:
            prid = msg['msg']['pullrequest']['id']
            username = msg['msg']['flag']['username']
            comment = msg['msg']['flag']['comment']
            tmpl = self._(
                '{username} updated the flags on {project}#{id} with: '
                '"{comment}"'
            )
            return tmpl.format(
                username=username, id=prid, comment=comment, project=project)
        elif 'pagure.git.receive' in msg['topic']:
            if 'commit' in msg['msg']:
                tmpl = self._('{user} pushed to {repo} ({branch}). "{summary}"')
                return _git_receive_v1(msg, tmpl, **config)
            else:
                tmpl = self._('{user} pushed {n_commits} to {repo} ({branch})')
                return _git_receive_v2(msg, tmpl)

        else:
            pass

    def secondary_icon(self, msg, **config):
        username = msg['msg'].get('agent')
        if username:
            return avatar_url(username)
        else:
            return None

    def usernames(self, msg, **config):
        username = msg['msg'].get('agent')
        if not username and 'pagure.git.receive' in msg['topic']:
            email = msg['msg']['commit']['email']
            username = email2fas(email, **config)

        if username:
            return set([username])
        else:
            return set([])

    def objects(self, msg, **config):
        try:
            project = _get_project(msg['msg'])
        except KeyError:
            try:
                project = _get_project(msg['msg']['pullrequest'])
            except KeyError:
                project = "(unknown)"

        if 'pagure.project' in msg['topic']:
            return set([
                'project/%s' % project,
            ])
        elif 'pagure.issue' in msg['topic']:
            return set([
                'issue/%s' % msg['msg']['issue']['id'],
                'project/%s' % project,
            ])
        elif 'pagure.pull-request' in msg['topic']:
            return set([
                'pull-request/%s' % msg['msg']['pullrequest']['id'],
                'project/%s' % project,
            ])
        elif 'pagure.git.receive' in msg['topic']:
            if 'commit' in msg['msg']:
                project = _get_project(msg['msg']['commit'], key='repo')
            else:
                project = _get_project(msg['msg'], key='repo')
            return set([
                'project/%s' % project,
            ])

        return set([])



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import \
        avatar_url_from_email, avatar_url_from_openid, email2fas

from fedmsg_meta_fedora_infrastructure.conglomerators.mailman import \
        mailman as mailman_conglomerator

import email.utils
import warnings

import fedmsg.config
config = fedmsg.config.load_config()


def _full_email_to_email(full_from):
    return email.utils.parseaddr(full_from)[1]


def _email_to_username(email):
    return email2fas(email, **config)


class MailmanProcessor(BaseProcessor):
    __name__ = "Mailman"
    __description__ = "mailing list messages"
    __link__ = "https://lists.fedoraproject.org"
    __docs__ = ("https://fedoraproject.org/wiki/"
                "Communicating_and_getting_help#Mailing_Lists")
    __obj__ = "Mailing List Messages"
    __icon__ = "http://cloud.ohloh.net/attachments/37686/mailman_med.png"

    conglomerators = [
        mailman_conglomerator.ByMessageId,
    ]

    def subtitle(self, msg, **config):
        if 'receive' in msg['topic']:
            lst = msg['msg']['mlist']['list_name']
            subject = msg['msg']['msg']['subject'].replace('\n', ' ')

            full_from = msg['msg']['msg']['from']
            user = _email_to_username(_full_email_to_email(full_from))
            if not user:
                user = "someone"

            d = msg['msg']['msg']
            if d['references'] or d['in-reply-to']:
                tmpl = self._(
                    "On the {lst} list, {user} replied to '{subject}'")
            else:
                tmpl = self._("{user} wrote '{subject}' to the {lst} list")

            return tmpl.format(lst=lst, user=user, subject=subject)
        else:
            warnings.warn("mailman3 message *must* have 'receive' in topic")

    def secondary_icon(self, msg, **config):
        full_from = msg['msg']['msg']['from']
        email = _full_email_to_email(full_from)

        # Can we find this person in FAS?
        username = email2fas(email, **config)

        if '@' in username:
            # No?  Then use their email for libravatar
            return avatar_url_from_email(email)
        else:
            # Yes?  Then use their openid like everywhere else.
            return avatar_url_from_openid(username)

    def link(self, msg, **config):
        base_url = 'https://lists.fedoraproject.org/archives'
        archived_at = msg['msg']['msg']['archived-at']
        if archived_at and archived_at.startswith('<'):
            archived_at = archived_at[1:]
        if archived_at and archived_at.endswith('>'):
            archived_at = archived_at[:-1]
        if archived_at and archived_at.startswith('http'):
            return archived_at
        elif archived_at:
            return base_url + archived_at
        else:
            return None

    def usernames(self, msg, **config):
        full_from = msg['msg']['msg']['from']
        user = _email_to_username(_full_email_to_email(full_from))
        if user and '@' not in user:
            return set([user])
        else:
            return set()

    def objects(self, msg, **config):

        # Build a repr of all the messages in this thread
        references = msg['msg']['msg']['references']

        # Fall back to this header if there's nothing in the first.
        if not references:
            references = msg['msg']['msg']['in-reply-to']

        references = references and references.split() or []
        references = [r[1:-1] for r in references]
        message_id = msg['msg']['msg']['message-id'][1:-1]

        if references:
            tokens = ['/'.join(references), message_id, 'message']
        else:
            tokens = [message_id, 'message']

        lst = msg['msg']['mlist']['list_name']
        tokens = [lst] + tokens

        return set(['/'.join(tokens)])



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Martin Krizek <mkrizek@redhat.com>
#           Ralph Bean <rbean@redhat.com>
#
""" Tests for taskotron messages """

import os
import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestTaskotronResultNew(Base):
    """ Taskotron emits messages on this topic anytime a task finishes.

    Here's an example message of a new **taskotron result** published about a
    **koji build**.
    """
    expected_title = "taskotron.result.new"
    expected_subti = "fedoratest PASSED for foobar-1.0-1.fc99"
    expected_link = ("https://taskotron.fedoraproject.org/taskmaster/"
                     "/builders/x86_64/builds/1/steps/runtask/logs/stdio")
    expected_secondary_icon = ("https://apps.fedoraproject.org/packages/"
                               "images/icons/foobar.png")
    expected_packages = set(['foobar'])
    expected_objects = set(['fedoratest/foobar/PASSED'])
    msg = {
        u'username': u'taskotron',
        u'i': 1,
        u'timestamp': 1389298195,
        u'msg_id': u'2014-10b5b1b6-42c7-4d64-aeae-5029b9515d47',
        u'topic': u'org.fedoraproject.stg.taskotron.result.new',
        u'msg': {
            u'task': {
                'name': 'fedoratest',
                'type': 'koji_build',
                'item': 'foobar-1.0-1.fc99',
            },
            u'result': {
                'id': '1',
                'submit_time': '2015-01-30 13:11:35.366862',
                'outcome': 'PASSED',
                'job_url': 'https://taskotron.fedoraproject.org/resultsdb/jobs/1',
                'log_url': ('https://taskotron.fedoraproject.org/taskmaster/'
                           '/builders/x86_64/builds/1/steps/runtask/logs/stdio'),
            },
        }
    }


class TestTaskotronBodhiUpdate(Base):
    """ Taskotron emits messages on this topic anytime a task finishes.

    Here's an example message of a new **taskotron result** published about a
    **bodhi update**.
    """
    expected_title = "taskotron.result.new"
    expected_subti = "upgradepath PASSED for FEDORA-2015-2f6c7508b7"
    expected_link = ("https://taskotron.fedoraproject.org/artifacts/"
                     "all/64981220-a418-11e5-91ee-52540053ee00/"
                     "task_output/FEDORA-2015-2f6c7508b7.log")
    msg = {
        "i": 177,
        "timestamp": 1450286061.0,
        "msg_id": "2015-2ff9d8af-8263-4886-8e3d-61cc7a5eeeb4",
        "topic": "org.fedoraproject.prod.taskotron.result.new",
        "msg": {
            "task": {
                "item": "FEDORA-2015-2f6c7508b7",
                "type": "bodhi_update",
                "name": "upgradepath"
            },
            "result": {
                "job_url": ("https://taskotron.fedoraproject.org/execdb/"
                            "/jobs/64981220-a418-11e5-91ee-52540053ee00"),
                "submit_time": "2015-12-16 17:14:21 UTC",
                "outcome": "PASSED",
                "id": 5202575,
                "log_url": ("https://taskotron.fedoraproject.org/artifacts/"
                            "all/64981220-a418-11e5-91ee-52540053ee00/"
                            "task_output/FEDORA-2015-2f6c7508b7.log"),
            }
        }
    }


#  only run these two tests if we have network connectivity.
#  (they'll fail in koji)
if not 'FEDMSG_META_NO_NETWORK' in os.environ:
    TestTaskotronBodhiUpdate.expected_secondary_icon = (
        "https://apps.fedoraproject.org/packages/"
        "images/icons/copy-jdk-configs.png")
    TestTaskotronBodhiUpdate.expected_packages = set(['copy-jdk-configs'])
    TestTaskotronBodhiUpdate.expected_objects = set([
        'upgradepath/copy-jdk-configs/PASSED'])


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
import six

from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url

try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict

from fedmsg_meta_fedora_infrastructure.conglomerators.pkgdb import \
        acls as pkgdb_acls

def get_agent(msg):
    """ Handy hack to handle legacy messages where 'agent' was a list.  """
    agent = msg['msg']['agent']
    if isinstance(agent, list):
        agent = agent[0]
    return agent


class PkgdbProcessor(BaseProcessor):
    __name__ = "Pkgdb"
    __description__ = "the Fedora Package DB"
    __link__ = "https://admin.fedoraproject.org/pkgdb"
    __docs__ = "https://fedorahosted.org/packagedb/"
    __obj__ = "Package ACL Updates"
    __icon__ = ("https://apps.fedoraproject.org/packages/"
                "images/icons/package_128x128.png")
    conglomerators = [
        pkgdb_acls.BySubject,
        pkgdb_acls.ByPackage,
        pkgdb_acls.ByAgent,
    ]

    def subtitle(self, msg, **config):
        if 'pkgdb.acl.update' in msg['topic']:
            tmpl = self._(
                u"{agent} changed {user}'s '{acl}' permission " +
                "on {package} ({branch}) to '{status}'"
            )
            user = msg['msg']['username']
            status = msg['msg']['status']
            package = msg['msg']['package_listing']['package']['name']
            acl = msg['msg']['acl']
            agent = get_agent(msg)
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(
                agent=agent, acl=acl,
                user=user, status=status,
                package=package, branch=branch)
        elif 'pkgdb.branch.clone' in msg['topic']:
            tmpl = self._(u"{agent} branched {package} {branch} from {master}")
            agent = get_agent(msg)
            package = msg['msg']['package']
            branch = msg['msg']['branch']
            master = msg['msg']['master']
            return tmpl.format(agent=agent, package=package,
                               branch=branch, master=master)
        elif 'pkgdb.package.update.status' in msg['topic']:
            tmpl = self._(u"{agent} {verb} {package}{extra}")
            extra = ""

            # prev_status, status and package_listing only apear in
            # pkgdb.package.update.status messages, but there are also
            # pgkdb.package.update messages
            prev_status = msg['msg'].get('prev_status')
            status = msg['msg'].get('status')
            if 'package_listing' in msg['msg']:
                branchname = msg['msg']['package_listing']['collection'][
                    'branchname']
                extra = self._(u" in {branchname}".format(
                    branchname=branchname))

            status_map = OrderedDict([
                ("Orphaned", [self._(u"unorphaned"), self._(u"orphaned")]),
                ("Retired", [self._(u"unretired"), self._(u"retired")]),
            ])
            verb = self._(u"made some updates to")
            for key, values in status_map.items():
                left, right = values
                if prev_status == key and status != key:
                    verb = left
                elif prev_status != key and status == key:
                    verb = right

            agent = get_agent(msg)

            try:
                package = msg['msg']['package_listing']['package']['name']
            except KeyError:
                package = msg['msg']['package']

            return tmpl.format(agent=agent, package=package, verb=verb,
                               extra=extra)
        elif msg['topic'].endswith('pkgdb.package.update'):
            agent = get_agent(msg)

            try:
                package = msg['msg']['package_listing']['package']['name']
            except KeyError:
                package = msg['msg']['package']
            if isinstance(package, dict):
                package = package['name']

            if 'fields' in msg['msg']:
                # New pkgdb2 style
                tmpl = self._(u"{agent} updated: {fields} of {package}")
                fields = ', '.join(msg['msg'].get('fields'))
                return tmpl.format(agent=agent, package=package, fields=fields)
            else:
                # old old pkgdb1 style
                tmpl = self._(u"{agent} made some updates to {package}")
                return tmpl.format(agent=agent, package=package)
        elif 'pkgdb.critpath.update' in msg['topic']:
            tmpl = self._(
                u"{agent} altered the critpath status for some packages")
            agent = get_agent(msg)
            return tmpl.format(agent=agent)
        elif msg['topic'].endswith('pkgdb.package.new'):
            tmpl = self._(
                u"{agent} added a new package '{package}' ({branch})")
            agent = get_agent(msg)
            package = msg['msg']['package_listing']['package']['name']
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(agent=agent, package=package, branch=branch)
        elif 'pkgdb.acl.request.toggle' in msg['topic']:
            tmpl = self._(
                u"{agent} has {action} '{acl}' on {package} ({branch})"
            )
            package = msg['msg']['package_listing']['package']['name']
            acl = msg['msg']['acl']
            agent = get_agent(msg)
            branch = msg['msg']['package_listing']['collection']['branchname']
            action = msg['msg']['acl_action']
            return tmpl.format(
                agent=agent, acl=acl, action=action,
                package=package, branch=branch)
        elif 'pkgdb.package.retire' in msg['topic']:
            tmpl = self._(
                u"{agent} {action} {package} ({branch})!"
            )
            package = msg['msg']['package_listing']['package']['name']
            action = msg['msg']['retirement']
            agent = get_agent(msg)
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(
                agent=agent, action=action,
                package=package, branch=branch)
        elif 'pkgdb.owner.update' in msg['topic']:
            tmpl = self._(
                u"{agent} changed owner of {package} ({branch}) to '{owner}'")

            # Owners got renamed to points of contact in packagedb2
            try:
                owner = msg['msg']['package_listing']['point_of_contact']
            except KeyError:
                owner = msg['msg']['package_listing']['owner']

            package = msg['msg']['package_listing']['package']['name']
            agent = get_agent(msg)
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(
                agent=agent,
                owner=owner,
                package=package,
                branch=branch)
        elif 'pkgdb.acl.user.remove' in msg['topic']:
            tmpl = self._(
                u"{agent} removed {user} from {package} ({branches})")
            package = msg['msg']['package_listings'][0]['package']['name']
            user = msg['msg']['username']
            agent = get_agent(msg)
            branches = ", ".join([
                p['collection']['branchname']
                for p in msg['msg']['package_listings']
            ])
            return tmpl.format(
                agent=agent,
                user=user,
                package=package,
                branches=branches)
        elif 'pkgdb.branch.start' in msg['topic']:
            tmpl = self._(
                u"{agent} started a branch of {tobranch} from {frombranch}")
            agent = get_agent(msg)
            frombranch = msg['msg']['collection_from']['branchname']
            tobranch = msg['msg']['collection_to']['branchname']
            return tmpl.format(
                agent=agent,
                frombranch=frombranch,
                tobranch=tobranch,
            )
        elif 'pkgdb.branch.complete' in msg['topic']:
            tmpl = self._(
                u"{agent}'s branch of {tobranch} from {frombranch} completed")
            agent = get_agent(msg)
            frombranch = msg['msg']['collection_from']['branchname']
            tobranch = msg['msg']['collection_to']['branchname']
            return tmpl.format(
                agent=agent,
                frombranch=frombranch,
                tobranch=tobranch,
            )
        elif 'pkgdb.collection.new' in msg['topic']:
            tmpl = self._(
                u"{agent} created a new collection for {name} {version}")
            agent = get_agent(msg)
            name = msg['msg']['collection']['name']
            version = msg['msg']['collection']['version']
            return tmpl.format(
                agent=agent,
                name=name,
                version=version,
            )
        elif 'pkgdb.collection.update' in msg['topic']:
            tmpl = self._(
                u"{agent} updated the following fields of the "
                "{name} {version} collection: {fields}")
            agent = get_agent(msg)
            name = msg['msg']['collection']['name']
            version = msg['msg']['collection']['version']
            fields = ", ".join(msg['msg']['fields'])
            return tmpl.format(
                agent=agent,
                name=name,
                version=version,
                fields=fields,
            )
        elif 'pkgdb.package.delete' in msg['topic']:
            tmpl = self._(
                u"{agent} deleted the '{package}' package from the pkgdb")
            agent = get_agent(msg)
            package = msg['msg']['package']['name']
            return tmpl.format(agent=agent, package=package)
        elif 'pkgdb.package.branch.delete' in msg['topic']:
            tmpl = self._(
                u"{agent} deleted the {branch} branch "
                "of the '{package}' package")
            agent = get_agent(msg)
            package = msg['msg']['package_listing']['package']['name']
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(agent=agent, branch=branch, package=package)
        elif msg['topic'].endswith('pkgdb.package.branch.new'):
            tmpl = self._(
                u"{agent} created the branch '{branch}' for the package "
                "'{package}'")
            agent = get_agent(msg)
            package = msg['msg']['package']['name']
            branch = msg['msg']['package_listing']['collection']['branchname']
            return tmpl.format(agent=agent, branch=branch, package=package)
        elif 'pkgdb.acl.delete' in msg['topic']:
            tmpl = self._(
                u"{agent} deleted {user}'s {acl} "
                "rights from {package} ({branch})")
            _msg = msg['msg']['acl']
            package = _msg['packagelist']['package']['name']
            branch = _msg['packagelist']['collection']['branchname']
            acl = _msg['acl']
            user = _msg['fas_name']
            agent = msg['msg']['agent']
            return tmpl.format(agent=agent, user=user, acl=acl,
                               package=package, branch=branch)
        elif msg['topic'].endswith('pkgdb.package.branch.request'):
            tmpl = self._(
                u"{agent} requested branch {new_branch} for package {package}")
            _msg = msg['msg']
            package = _msg['package']['name']
            new_branch = _msg['collection_to']['branchname']
            agent = msg['msg']['agent']
            return tmpl.format(agent=agent, new_branch=new_branch,
                               package=package)
        elif msg['topic'].endswith('pkgdb.package.new.request'):
            tmpl = self._(
                u"{agent} requested package {package} on branch {branch}")
            _msg = msg['msg']
            package = _msg['info']['pkg_name']
            branch = _msg['collection']['branchname']
            agent = msg['msg']['agent']
            return tmpl.format(agent=agent, branch=branch, package=package)
        elif msg['topic'].endswith('pkgdb.admin.action.status.update'):
            tmpl = self._(
                u"{agent} changed {owner}'s {action} "
                "for {package} in {branch} "
                "from {old_status} to {new_status}")
            _msg = msg['msg']

            agent = _msg['agent']
            owner = _msg['action']['user']

            branch = _msg['action']['collection']['branchname']
            try:
                package = _msg['action']['info']['pkg_name']
            except KeyError:
                package = _msg['action']['package']['name']

            action = _msg['action']['action']
            substitutes = {
                'request.package': self._('package request'),
                'request.branch': self._('branch request'),
            }
            action = substitutes.get(action, action)

            old_status = _msg['old_status']
            new_status = _msg['new_status']

            message = _msg['action'].get('message', None)
            if message:
                tmpl += self._(" with message: {message}")

            return tmpl.format(
                agent=agent, owner=owner, action=action,
                package=package, branch=branch,
                old_status=old_status, new_status=new_status, message=message)
        elif msg['topic'].endswith('pkgdb.package.critpath.update'):
            tmpl = self._(
                u"{agent} {action} the critpath flag on the "
                "{package} package ({branches})")
            agent = msg['msg']['agent']
            action = ['unset', 'set'][msg['msg']['critpath']]
            package = msg['msg']['package']['name']
            branches = ', '.join(msg['msg']['branches'])
            return tmpl.format(agent=agent, action=action,
                               package=package, branches=branches)
        elif msg['topic'].endswith('pkgdb.package.monitor.update'):
            tmpl = self._(
                u"{agent} set the monitor flag of {package} to {status}")
            agent = msg['msg']['agent']
            status = msg['msg']['status']
            package = msg['msg']['package']['name']
            return tmpl.format(agent=agent, status=status, package=package)
        elif msg['topic'].endswith('pkgdb.package.unretire.request'):
            tmpl = self._(
                u"{agent} asks that {package} be unretired on {branch}")
            agent = msg['msg']['agent']
            branch = msg['msg']['collection']['branchname']
            package = msg['msg']['package']['name']
            return tmpl.format(agent=agent, branch=branch, package=package)
        elif msg['topic'].endswith('pkgdb.package.koschei.update'):
            tmpl = self._(
                u"{agent} set the koschei monitoring flag of {package} to "
                "{status}")
            agent = msg['msg']['agent']
            status = msg['msg']['status']
            package = msg['msg']['package']['name']
            return tmpl.format(agent=agent, status=status, package=package)
        else:
            raise NotImplementedError("%r" % msg)

    def secondary_icon(self, msg, **config):
        user = None

        try:
            user = get_agent(msg)
        except KeyError:
            pass

        try:
            user = msg['msg']['username']
        except KeyError:
            pass

        if not user:
            return ""

        return avatar_url(username=user)

    def usernames(self, msg, **config):
        users = set()

        try:
            users.add(get_agent(msg))
        except KeyError:
            pass

        try:
            users.add(msg['msg']['acl']['fas_name'])
        except (KeyError, TypeError):
            pass

        try:
            users.add(msg['msg']['username'])
        except KeyError:
            pass

        if 'orphan' in users:
            users.remove('orphan')

        return users

    def objects(self, msg, **config):
        objs = set()

        _msg = msg['msg']

        if 'pkgdb.acl.update' in msg['topic']:
            objs.add('{package}/acls/{branch}/{acl}/{user}'.format(
                package=_msg['package_listing']['package']['name'],
                branch=_msg['package_listing']['collection']['branchname'],
                acl=_msg['acl'],
                user=_msg['username']
            ))
        elif msg['topic'].endswith('pkgdb.package.new'):
            objs.add('{package}/create'.format(
                package=_msg['package_listing']['package']['name'],
            ))
        elif 'pkgdb.acl.request.toggle' in msg['topic']:
            objs.add('{package}/acls/{branch}/{acl}/{user}'.format(
                package=_msg['package_listing']['package']['name'],
                branch=_msg['package_listing']['collection']['branchname'],
                acl=_msg['acl'],
                user=_msg['agent']
            ))
        elif 'pkgdb.package.retire' in msg['topic']:
            objs.add('{package}/retire'.format(
                package=_msg['package_listing']['package']['name'],
            ))
        elif 'pkgdb.acl.user.remove' in msg['topic']:
            objs.add('{package}/remove/{user}'.format(
                package=_msg['package_listings'][0]['package']['name'],
                user=_msg['username'],
            ))
        elif 'pkgdb.owner.update' in msg['topic']:
            objs.add('{package}/owner/{branch}'.format(
                package=_msg['package_listing']['package']['name'],
                branch=_msg['package_listing']['collection']['branchname'],
            ))
        elif 'pkgdb.package.update' in msg['topic']:
            try:
                package = _msg['package_listing']['package']['name']
            except KeyError:
                package = _msg['package']
            if isinstance(package, dict):
                package = package['name']
            objs.add('{package}/update'.format(package=package))
        elif 'pkgdb.branch.clone' in msg['topic']:
            objs.add('{package}/branch'.format(package=_msg['package']))
        elif 'pkgdb.package.delete' in msg['topic']:
            package = _msg['package']['name']
            objs.add('{package}/package/delete'.format(package=package))
        elif 'pkgdb.package.branch.delete' in msg['topic']:
            package = _msg['package_listing']['package']['name']
            branch = _msg['package_listing']['collection']['branchname']
            objs.add('{package}/{branch}/delete'.format(
                package=package, branch=branch))
        elif 'pkgdb.package.branch.new' in msg['topic']:
            package = _msg['package']['name']
            branch = _msg['package_listing']['collection']['branchname']
            objs.add('{package}/{branch}/new'.format(
                package=package, branch=branch))
        elif 'pkgdb.acl.delete' in msg['topic']:
            objs.add('{package}/acls/{branch}/{acl}/{user}'.format(
                package=_msg['acl']['packagelist']['package']['name'],
                branch=_msg['acl']['packagelist']['collection']['branchname'],
                acl=_msg['acl']['acl'],
                user=_msg['acl']['fas_name'],
            ))
        elif msg['topic'].endswith('pkgdb.package.branch.request'):
            objs.add('{package}/branch/request/{branch}/{user}'.format(
                package=_msg['package']['name'],
                branch=_msg['collection_to']['branchname'],
                user=_msg['agent'],
            ))
        elif msg['topic'].endswith('pkgdb.package.new.request'):
            objs.add('new/package/request/{package}/{branch}/{user}'.format(
                package=_msg['info']['pkg_name'],
                branch=_msg['collection']['branchname'],
                user=_msg['agent'],
            ))
        elif msg['topic'].endswith('pkgdb.admin.action.status.update'):
            try:
                package = _msg['action']['info']['pkg_name']
            except KeyError:
                package = _msg['action']['package']['name']
            objs.add(
                'action/{actionid}/status/{package}/{branch}/{user}'.format(
                    actionid=_msg['action']['id'],
                    package=package,
                    branch=_msg['action']['collection']['branchname'],
                    user=_msg['agent'],
                )
            )
        elif msg['topic'].endswith('pkgdb.package.critpath.update'):
            objs.add(_msg['package']['name'] + "/critpath")
        elif msg['topic'].endswith('pkgdb.package.monitor.update'):
            objs.add(
                '{package}/monitor/{status}'.format(
                package=_msg['package']['name'],
                status=str(_msg['status']).lower())
            )
        elif msg['topic'].endswith('pkgdb.package.koschei.update'):
            objs.add(
                '{package}/koschei/{status}'.format(
                package=_msg['package']['name'],
                status=str(_msg['status']).lower())
            )
        elif msg['topic'].endswith('pkgdb.package.unretire.request'):
            objs.add(
                '{package}/unretire/{branch}'.format(
                package=_msg['package']['name'],
                branch=msg['msg']['collection']['branchname'])
            )

        return objs

    def packages(self, msg, **config):
        packages = set()

        try:
            packages.add(msg['msg']['package_listing']['package']['name'])
        except KeyError:
            pass

        try:
            for package in msg['msg']['package_listings']:
                packages.add(package['package']['name'])
        except KeyError:
            pass

        try:
            if isinstance(msg['msg']['package'], six.string_types):
                packages.add(msg['msg']['package'])
            else:
                packages.add(msg['msg']['package']['name'])
        except (KeyError, TypeError):
            pass

        try:
            packages.add(msg['msg']['acl']['packagelist']['package']['name'])
        except (KeyError, TypeError):
            pass

        try:
            packages.add(msg['msg']['info']['pkg_name'])
        except (KeyError, TypeError):
            pass

        try:
            packages.add(msg['msg']['action']['info']['pkg_name'])
        except (KeyError, TypeError):
            pass

        try:
            packages.add(msg['msg']['action']['package']['name'])
        except (KeyError, TypeError):
            pass

        return packages

    def link(self, msg, **config):
        tmpl = "https://admin.fedoraproject.org/pkgdb/package/{package}/"

        if any(map(msg['topic'].__contains__, [
            'pkgdb.package.new.request',
        ])):
            return tmpl.format(
                package=msg['msg']['info']['pkg_name']
            )


        if any(map(msg['topic'].__contains__, [
            'pkgdb.admin.action.status.update',
        ])):
            try:
                package = msg['msg']['action']['info']['pkg_name']
            except KeyError:
                package = msg['msg']['action']['package']['name']
            return tmpl.format(package=package)

        if any(map(msg['topic'].__contains__, [
            'pkgdb.acl.update',
            'pkgdb.acl.request.toggle',
            'pkgdb.owner.update',
            'pkgdb.package.retire',
            'pkgdb.package.new',
            'pkgdb.package.branch.new',
        ])):
            return tmpl.format(
                package=msg['msg']['package_listing']['package']['name']
            )

        if any(map(msg['topic'].__contains__, [
            'pkgdb.acl.user.remove',
        ])):
            return tmpl.format(
                package=msg['msg']['package_listings'][0]['package']['name']
            )

        if any(map(msg['topic'].__contains__, [
            'pkgdb.package.update',
            'pkgdb.branch.clone',
            'package.branch.request',
        ])):
            try:
                package = msg['msg']['package_listing']['package']['name']
            except KeyError:
                package = msg['msg']['package']
            if isinstance(package, dict):
                package = package['name']
            return tmpl.format(package=package)

        return ""



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Pierre-Yves Chibon <pingou@pingoured.fr>
#           Ralph Bean <rbean@redhat.com>
#
""" Tests for mdapi messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestNewProjectLegacy(Base):
    """ These are what the messages used to look like... """
    expected_title = "mdapi.repo.update"
    expected_subti = (
        'mdapi noticed a rawhide repomd change: '
        'conflicts: +1/-2, '
        'files: +5/-4, '
        'obsoletes: +2/-2, '
        'packages: +2/-3, '
        'provides: +2/-2, '
        'recommends: +2/-1, '
        'requires: +2/-2, '
        'suggests: +2/-2')
    expected_link = "https://apps.fedoraproject.org/mdapi"
    expected_link = "https://download.fedoraproject.org/pub/fedora/linux/" + \
        "development/rawhide/x86_64/os/repodata/" + \
        "a0c75ee3abe48a4b602560d1c7c408d8d151caa1865678d2e0e1eb442d96ba90-" + \
        "filelists.sqlite.xz"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_packages = set([
        'java-1.8.0-openjdk-devel-debug',
        'vips',
        'python3-celery',
        'python3-dropbox',
        'libfdisk',
        'journal-brief',
        'meataxe',
        'rpm',
        'akonadi-mysql',
        'java-1.8.0-openjdk-headless',
        'python-wrapt-doc',
        'python3-yui',
        'python-faker-doc',
        'python3-PyQt4-devel',
        'boost',
        'PyOpenGL',
        'audit-libs-devel',
        'python-simplegeneric',
        'python-aniso8601',
        'python3',
        'django-markdown2',
        'python-pytools',
        'tyrus-container-servlet',
        'python3-devel',
        'waf-python3',
        'avahi-ui-sharp-devel',
        'python2-XStatic-roboto-fontface',
        'python-Traits',
    ])
    expected_usernames = set([])
    expected_objects = set([
        'rawhide/recommends/added/journal-brief',
        'rawhide/files/removed/python3-devel',
        'rawhide/provides/added/python2-XStatic-roboto-fontface',
        'rawhide/files/removed/python3-celery',
        'rawhide/provides/added/avahi-ui-sharp-devel(x86-64)',
        'rawhide/packages/removed/python-wrapt-doc',
        'rawhide/files/added/python3',
        'rawhide/files/added/java-1.8.0-openjdk-headless',
        'rawhide/obsoletes/removed/python-Traits(x86-64)',
        'rawhide/requires/removed/rpm',
        'rawhide/packages/removed/python3-dropbox',
        'rawhide/suggests/added/meataxe',
        'rawhide/recommends/removed/journal-brief',
        'rawhide/provides/removed/django-markdown2',
        'rawhide/conflicts/added/vips',
        'rawhide/files/removed/waf-python3',
        'rawhide/requires/added/libfdisk',
        'rawhide/recommends/added/akonadi-mysql',
        'rawhide/packages/added/python3-PyQt4-devel',
        'rawhide/obsoletes/added/python-pytools',
        'rawhide/obsoletes/added/python-aniso8601',
        'rawhide/conflicts/removed/vips',
        'rawhide/suggests/added/python-faker-doc',
        'rawhide/suggests/removed/python-faker-doc',
        'rawhide/requires/removed/boost(x86-32)',
        'rawhide/files/added/java-1.8.0-openjdk-devel-debug',
        'rawhide/packages/removed/audit-libs-devel',
        'rawhide/provides/removed/python3-yui(x86-64)',
        'rawhide/files/removed/java-1.8.0-openjdk-headless',
        'rawhide/requires/added/PyOpenGL',
        'rawhide/files/added/python3-celery',
        'rawhide/conflicts/removed/python3',
        'rawhide/packages/added/tyrus-container-servlet',
        'rawhide/obsoletes/removed/python-simplegeneric',
    ])
    msg = {
        "source_name": "datanommer",
        "i": 3,
        "timestamp": 1447413644.0,
        "msg_id": "2015-dbf52782-0a87-448f-a6fb-20c506cdd4eb",
        "topic": "org.fedoraproject.stg.mdapi.repo.update",
        "source_version": "0.6.5",
        "msg": {
          "url": "fedora/linux/development/rawhide/x86_64/os/repodata/"
            "a0c75ee3abe48a4b602560d1c7c408d8d151caa1865678d2e0e1eb442d96ba90"
            "-filelists.sqlite.xz",
          "differences": {
            "files": {
              "removed": [
                [
                  "/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.60-17.b28.fc24.x86_64/jre/bin/tnameserv",
                  "file",
                  "java-1.8.0-openjdk-headless"
                ],
                [
                  "/usr/bin/python3.4m-x86_64-config",
                  "file",
                  "python3-devel"
                ],
                [
                  "/usr/bin/waf-3.4",
                  "file",
                  "waf-python3"
                ],
                [
                  "/usr/lib/python3.4/site-packages/celery/tests/bin/__pycache__/test_beat.cpython-34.pyc",
                  "file",
                  "python3-celery"
                ]
              ],
              "added": [
                [
                  "/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.65-1.b17.fc24.i386/jre/bin/orbd",
                  "file",
                  "java-1.8.0-openjdk-headless"
                ],
                [
                  "/usr/lib/python3.5/site-packages/celery/bin/__pycache__/__init__.cpython-35.pyc",
                  "file",
                  "python3-celery"
                ],
                [
                  "/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.65-1.b17.fc24.x86_64-debug/bin/rmiregistry",
                  "file",
                  "java-1.8.0-openjdk-devel-debug"
                ],
                [
                  "/usr/lib/python3.5/site-packages/celery/tests/bin/test_celeryd_detach.py",
                  "file",
                  "python3-celery"
                ],
                [
                  "/usr/bin/pydoc3.5",
                  "file",
                  "python3"
                ]
              ]
            },
            "enhances": {
              "removed": [],
              "added": []
            },
            "recommends": {
              "removed": [
                [
                  "journal-brief",
                  "EQ",
                  "0",
                  "1.1.3",
                  "1.fc24",
                  "python3-journal-brief"
                ]
              ],
              "added": [
                [
                  "akonadi-mysql",
                  "EQ",
                  "0",
                  "1.13.0",
                  "22.fc24",
                  "akonadi"
                ],
                [
                  "journal-brief",
                  "EQ",
                  "0",
                  "1.1.3",
                  "2.fc24",
                  "python3-journal-brief"
                ]
              ]
            },
            "suggests": {
              "removed": [
                [
                  "python-faker-doc",
                  "EQ",
                  "0",
                  "0.5.3",
                  "4.fc24",
                  "python3-faker"
                ],
                [
                  "python-faker-doc",
                  "EQ",
                  "0",
                  "0.5.3",
                  "4.fc24",
                  "python2-faker"
                ]
              ],
              "added": [
                [
                  "python-faker-doc",
                  "EQ",
                  "0",
                  "0.5.3",
                  "5.fc24",
                  "python3-faker"
                ],
                [
                  "meataxe",
                  None,
                  None,
                  None,
                  None,
                  "gap-pkg-atlasrep"
                ],
              ]
            },
            "supplements": {
              "removed": [],
              "added": []
            },
            "obsoletes": {
              "removed": [
                [
                  "python-simplegeneric",
                  "LT",
                  "0",
                  "0.8.1",
                  "1.fc24",
                  "python2-simplegeneric"
                ],
                [
                  "python-Traits(x86-64)",
                  "LT",
                  "0",
                  "4.5.0",
                  "7.gitac5d029.fc24",
                  "python2-Traits"
                ],
              ],
              "added": [
                [
                  "python-pytools",
                  "LT",
                  "0",
                  "2015.1.2",
                  "2.fc24",
                  "python3-pytools"
                ],
                [
                  "python-aniso8601",
                  "LT",
                  "0",
                  "1.1.0",
                  "2.fc24",
                  "python2-aniso8601"
                ]
              ]
            },
            "provides": {
              "removed": [
                [
                  "django-markdown2",
                  "EQ",
                  "0",
                  "0.2.1",
                  "1.fc23",
                  "python-django-markdown2"
                ],
                [
                  "python3-yui(x86-64)",
                  "EQ",
                  "0",
                  "1.1.0",
                  "14.fc24",
                  "python3-yui"
                ]
              ],
              "added": [
                [
                  "avahi-ui-sharp-devel(x86-64)",
                  "EQ",
                  "0",
                  "0.6.32",
                  "0.3.rc.fc24",
                  "avahi-ui-sharp-devel"
                ],
                [
                  "python2-XStatic-roboto-fontface",
                  "EQ",
                  "0",
                  "0.4.3.2",
                  "5.fc24",
                  "python2-XStatic-roboto-fontface"
                ]
              ]
            },
            "conflicts": {
              "removed": [
                [
                  "vips",
                  "GT",
                  "0",
                  "8.1.1",
                  "1.fc24",
                  "vips-doc"
                ],
                [
                  "python3",
                  "LT",
                  "0",
                  "3.4.3",
                  "5.fc24",
                  "python3-devel"
                ]
              ],
              "added": [
                [
                  "vips",
                  "GT",
                  "0",
                  "8.1.1",
                  "2.fc24",
                  "vips-doc"
                ]
              ]
            },
            "packages": {
              "removed": [
                [
                  "audit-libs-devel",
                  "2.4.4",
                  "2.fc24",
                  "0",
                  "i686"
                ],
                [
                  "python3-dropbox",
                  "3.22",
                  "1.fc24",
                  "0",
                  "noarch"
                ],
                [
                  "python-wrapt-doc",
                  "1.10.5",
                  "1.fc23",
                  "0",
                  "x86_64"
                ]
              ],
              "added": [
                [
                  "tyrus-container-servlet",
                  "1.12",
                  "3.fc24",
                  "0",
                  "noarch"
                ],
                [
                  "python3-PyQt4-devel",
                  "4.11.4",
                  "5.fc24",
                  "0",
                  "i686"
                ]
              ]
            },
            "requires": {
              "removed": [
                [
                  "boost(x86-32)",
                  "EQ",
                  "0",
                  "1.59.0",
                  "6.fc24",
                  "boost-devel"
                ],
                [
                  "rpm",
                  "EQ",
                  "0",
                  "4.13.0",
                  "0.rc1.11.fc24",
                  "rpm-cron"
                ]
              ],
              "added": [
                [
                  "PyOpenGL",
                  "EQ",
                  "0",
                  "3.1.0",
                  "4.fc24",
                  "PyOpenGL-Tk"
                ],
                [
                  "libfdisk",
                  "EQ",
                  "0",
                  "2.27.1",
                  "2.fc24",
                  "libfdisk-devel"
                ]
              ]
            }
          },
          "name": "rawhide"
        }
      }


class TestNewProject(Base):
    """ These messages are published when a new project is created on
    `pagure <https://pagure.io>`_.
    """
    expected_title = "mdapi.repo.update"
    expected_subti = (
        'mdapi noticed a rawhide repomd change: '
        'PyOpenGL, akonadi-mysql, and 26 others'
    )
    expected_link = "https://apps.fedoraproject.org/mdapi"
    expected_link = "https://download.fedoraproject.org/pub/fedora/linux/" + \
        "development/rawhide/x86_64/os/repodata/" + \
        "a0c75ee3abe48a4b602560d1c7c408d8d151caa1865678d2e0e1eb442d96ba90-" + \
        "filelists.sqlite.xz"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_packages = set([
        'java-1.8.0-openjdk-devel-debug',
        'vips',
        'python3-celery',
        'python3-dropbox',
        'libfdisk',
        'journal-brief',
        'meataxe',
        'rpm',
        'akonadi-mysql',
        'java-1.8.0-openjdk-headless',
        'python-wrapt-doc',
        'python3-yui',
        'python-faker-doc',
        'python3-PyQt4-devel',
        'boost',
        'PyOpenGL',
        'audit-libs-devel',
        'python-simplegeneric',
        'python-aniso8601',
        'python3',
        'django-markdown2',
        'python-pytools',
        'tyrus-container-servlet',
        'python3-devel',
        'waf-python3',
        'avahi-ui-sharp-devel',
        'python2-XStatic-roboto-fontface',
        'python-Traits',
    ])
    expected_usernames = set([])
    expected_objects = set([
        'rawhide/java-1.8.0-openjdk-devel-debug',
        'rawhide/vips',
        'rawhide/python3-celery',
        'rawhide/python3-dropbox',
        'rawhide/libfdisk',
        'rawhide/journal-brief',
        'rawhide/meataxe',
        'rawhide/rpm',
        'rawhide/akonadi-mysql',
        'rawhide/java-1.8.0-openjdk-headless',
        'rawhide/python-wrapt-doc',
        'rawhide/python3-yui',
        'rawhide/python-faker-doc',
        'rawhide/python3-PyQt4-devel',
        'rawhide/boost',
        'rawhide/PyOpenGL',
        'rawhide/audit-libs-devel',
        'rawhide/python-simplegeneric',
        'rawhide/python-aniso8601',
        'rawhide/python3',
        'rawhide/django-markdown2',
        'rawhide/python-pytools',
        'rawhide/tyrus-container-servlet',
        'rawhide/python3-devel',
        'rawhide/waf-python3',
        'rawhide/avahi-ui-sharp-devel',
        'rawhide/python2-XStatic-roboto-fontface',
        'rawhide/python-Traits',
    ])
    msg = {
        "source_name": "datanommer",
        "i": 3,
        "timestamp": 1447413644.0,
        "msg_id": "2015-dbf52782-0a87-448f-a6fb-20c506cdd4eb",
        "topic": "org.fedoraproject.stg.mdapi.repo.update",
        "source_version": "0.6.5",
        "msg": {
          "url": "fedora/linux/development/rawhide/x86_64/os/repodata/"
            "a0c75ee3abe48a4b602560d1c7c408d8d151caa1865678d2e0e1eb442d96ba90"
            "-filelists.sqlite.xz",
          "packages": [
            'java-1.8.0-openjdk-devel-debug',
            'vips',
            'python3-celery',
            'python3-dropbox',
            'libfdisk',
            'journal-brief',
            'meataxe',
            'rpm',
            'akonadi-mysql',
            'java-1.8.0-openjdk-headless',
            'python-wrapt-doc',
            'python3-yui',
            'python-faker-doc',
            'python3-PyQt4-devel',
            'boost',
            'PyOpenGL',
            'audit-libs-devel',
            'python-simplegeneric',
            'python-aniso8601',
            'python3',
            'django-markdown2',
            'python-pytools',
            'tyrus-container-servlet',
            'python3-devel',
            'waf-python3',
            'avahi-ui-sharp-devel',
            'python2-XStatic-roboto-fontface',
            'python-Traits',
          ],
          "name": "rawhide"
        }
      }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



import fedmsg.meta.base
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url


class ByURL(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return '.meetbot.' in msg['topic']

    def get_title(self, ms):
        for msg in ms:
            title = msg['msg'].get('meeting_topic')
            if title:
                return title
        return None

    def merge(self, constituents, subject, **config):
        ms = constituents  # shorthand
        usernames = self.list_to_series(
            sum([list(m['msg']['attendees'].keys()) for m in ms], [])
        )
        channel = ms[0]['msg']['channel']
        title = self.get_title(ms)

        if title:
            subtitle = self._('{usernames} participated in {title} in {channel}')
        else:
            subtitle = self._('{usernames} participated in a meeting in {channel}')

        tmpl = self.produce_template(constituents, subject, **config)
        tmpl['subtitle'] = subtitle.format(
            usernames=usernames,
            title=title,
            channel=channel,
        )
        tmpl['subjective'] = tmpl['subtitle']

        default = tmpl['icon']

        # These are the only two keys that vary amongst our concrete children.
        tmpl['secondary_icon'] = self.get_secondary_icon(constituents, default)
        tmpl['link'] = self.get_link(constituents)

        return tmpl

    def matches(self, a, b, **config):
        """ The events must be all about the same meetbot """
        a, b = a['msg'], b['msg']
        if a['url'] != b['url']:
            return False
        return True

    def get_secondary_icon(self, constituents, default):
        username = constituents[0]['msg']['owner']
        return avatar_url(username)

    def get_link(self, constituents):
        return constituents[0]['msg']['url']



# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for fedmsg.meta """

import os
import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from fedmsg_meta_fedora_infrastructure.tests.common import add_doc


full_irc_logs = """======================================
#fedora-meeting-1: RELENG (2015-02-09)
======================================


Meeting started by dgilmore at 16:31:17 UTC. The full logs are available
at
http://meetbot.fedoraproject.org/fedora-meeting-1/2015-02-09/releng.2015-02-09-16.31.log.html
.



Meeting summary
---------------
* init process  (dgilmore, 16:31:30)

* Secondary Architectures updates  (dgilmore, 16:33:42)

* Secondary Architectures update - ppc  (dgilmore, 16:33:43)

* Secondary Architectures update - s390  (dgilmore, 16:36:35)

* Secondary Architectures update - arm  (dgilmore, 16:37:12)

* #6016 Use fedpkg-minimal in Fedora buildroots  (dgilmore, 16:39:22)
  * LINK: https://fedorahosted.org/rel-eng/ticket/6016   (dgilmore,
    16:39:38)
  * LINK: https://koji.fedoraproject.org/koji/buildinfo?buildID=609961
    (bochecha, 16:40:40)

* #5959 Enable keep-alive connections for koji (primary and secondaries)
  (dgilmore, 16:41:44)
  * LINK: https://fedorahosted.org/rel-eng/ticket/5959   (dgilmore,
    16:41:56)
  * ACTION: pbrobinson to send patch to infra list enabling keepalive
    (dgilmore, 16:43:08)

* #6023 allow Peter Robinson to restart sigul bridges  (dgilmore,
  16:43:39)
  * LINK: https://fedorahosted.org/rel-eng/ticket/6023   (dgilmore,
    16:43:48)

* #6027 secondary arch old mash trees cleanup  (dgilmore, 16:45:31)
  * LINK: https://fedorahosted.org/rel-eng/ticket/6027   (dgilmore,
    16:45:47)

* #6096 add individual email addressses to Fedoras GPG keys  (dgilmore,
  16:52:06)
  * LINK: https://fedorahosted.org/rel-eng/ticket/6096   (dgilmore,
    16:52:20)
  * ACTION: dgilmore will make a choice when making the keys  (dgilmore,
    16:58:51)

* #6100 create bsd-style checksum files for CHECKSUM files  (dgilmore,
  16:59:11)
  * LINK: https://fedorahosted.org/rel-eng/ticket/6100   (dgilmore,
    16:59:22)
  * ACTION: dgilmore to make this happen  (dgilmore, 17:03:13)

* open floor  (dgilmore, 17:03:21)

Meeting ended at 17:58:27 UTC.




Action Items
------------
* pbrobinson to send patch to infra list enabling keepalive
* dgilmore will make a choice when making the keys
* dgilmore to make this happen




Action Items, by person
-----------------------
* dgilmore
  * dgilmore will make a choice when making the keys
  * dgilmore to make this happen
* pbrobinson
  * pbrobinson to send patch to infra list enabling keepalive
* **UNASSIGNED**
  * (none)




People Present (lines said)
---------------------------
* dgilmore (116)
* bochecha (33)
* tyll (16)
* pbrobinson (16)
* zodbot (5)
* janeznemanic (1)
* masta (1)
* nirik (0)
* sharkcz (0)




Generated by `MeetBot`_ 0.1.4

.. _`MeetBot`: http://wiki.debian.org/MeetBot
"""

class TestSupybotStartMeetingNoName(Base):
    """ Trusty old `zodbot <https://meetbot.fedoraproject.org/>`_ publishes
    messages too!  Messages on this topic get published (somewhat obviously)
    when a new IRC meeting is started.  The user starting the meeting may
    specify a meeting title, but doesn't have to.  Here's an example
    message with no meeting title specified:
    """
    expected_title = "meetbot.meeting.start"
    expected_subti = 'ralph started a meeting in #channel'
    expected_icon = 'https://apps.fedoraproject.org/img/icons/meetbot.png'
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro"
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": None,
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel"
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.start",
        "timestamp": 1345572862.556145
    }


class TestSupybotStartMeeting(Base):
    """ Trusty old `zodbot <https://meetbot.fedoraproject.org/>`_ publishes
    messages too!  Messages on this topic get published (somewhat obviously)
    when a new IRC meeting is started.  The user starting the meeting may
    specify a meeting title, but doesn't have to.  Here's an example
    message with a specified meeting title:
    """
    expected_title = "meetbot.meeting.start"
    expected_subti = 'ralph started meeting "title" in #channel'
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
        'titles/title',
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": "title",
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel"
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.start",
        "timestamp": 1345572862.556145
    }


class TestSupybotEndMeeting(Base):
    """ Trusty old `zodbot <https://meetbot.fedoraproject.org/>`_ publishes
    messages too!  Messages on this topic get published when an IRC meeting
    ends.  Meetings may or may not have a title (which can be tricky).
    Here's an example message where the title is specified:
    """
    expected_title = "meetbot.meeting.complete"
    expected_subti = 'dgilmore\'s meeting titled "RELENG (2015-02-09)" ' + \
        'ended in #fedora-meeting-1'
    expected_link = 'https://meetbot.fedoraproject.org/fedora-meeting-1/' + \
        '2015-02-09/releng.2015-02-09-16.31.html'
    expected_usernames = set([
        'masta',
        'nirik',
        'tyll',
        'bochecha',
        'pbrobinson',
        'sharkcz',
        'janeznemanic',
        'dgilmore',
    ])
    expected_objects = set([
        'attendees/masta',
        'attendees/nirik',
        'attendees/tyll',
        'attendees/bochecha',
        'attendees/pbrobinson',
        'attendees/sharkcz',
        'attendees/janeznemanic',
        'attendees/dgilmore',
        'channels/#fedora-meeting-1',
        'titles/RELENG (2015-02-09)',
        'topics/open floor',
    ])

    msg = {
        "source_name": "datanommer",
        "i": 94,
        "timestamp": 1423504707.0,
        "msg_id": "2015-ff043996-5aaa-471b-bc87-b8551f7fd3b6",
        "topic": "org.fedoraproject.prod.meetbot.meeting.complete",
        "source_version": "0.6.4",
        "msg": {
            "meeting_topic": "RELENG (2015-02-09)",
            "attendees": {
                "masta": 1,
                "nirik": 0,
                "tyll": 16,
                "zodbot": 5,
                "bochecha": 33,
                "pbrobinson": 16,
                "sharkcz": 0,
                "janeznemanic": 1,
                "dgilmore": 116
            },
            "chairs": {
                "masta": True,
                "nirik": True,
                "tyll": True,
                "bochecha": True,
                "pbrobinson": True,
                "dgilmore": True,
                "sharkcz": True
            },
            "topic": "open floor",
            "url": "http://meetbot.fedoraproject.org/fedora-meeting-1/2015-02-09/releng.2015-02-09-16.31",
            "owner": "dgilmore",
            "channel": "#fedora-meeting-1"
        }
    }


class TestSupybotEndMeetingNoTitle(Base):
    """ Trusty old `zodbot <https://meetbot.fedoraproject.org/>`_ publishes
    messages too!  Messages on this topic get published when an IRC meeting
    ends.  Meetings may or may not have a title (which can be tricky).
    Here's an example message where the title is **not** specified:
    """
    expected_title = "meetbot.meeting.complete"
    expected_subti = 'ralph\'s meeting ended in #channel'
    expected_link = 'https://logs.com/awesome.html'
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": None,
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel"
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.complete",
        "timestamp": 1345572862.556145
    }


class TestSupybotChangeTopic(Base):
    """ As IRC meetings chug along, the chairperson may change the meeting;
    zodbot publishes message for that!  An example **with** a title specified:
    """
    expected_title = "meetbot.meeting.topic.update"
    expected_subti = 'The topic of ralph\'s "title" meeting changed ' +\
        'to "Food" in #channel'
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
        'titles/title',
        'topics/Food',
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": "title",
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel",
            "topic": "Food",
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.topic.update",
        "timestamp": 1345572862.556145
    }


class TestSupybotChangeTopicNoTitle(Base):
    """ As IRC meetings chug along, the chairperson may change the meeting;
    zodbot publishes message for that!  An example **without** a title
    specified:
    """
    expected_title = "meetbot.meeting.topic.update"
    expected_subti = 'The topic of ralph\'s meeting changed ' +\
        'to "Food" in #channel'
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
        'topics/Food'
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": None,
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel",
            "topic": "Food",
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.topic.update",
        "timestamp": 1345572862.556145
    }


class TestSupybotMeetingItemHelp(Base):
    """ As IRC meetings chug along, people can declare items for the record.
    Here's an example of someone **calling for help** on some item:
    """
    expected_title = "meetbot.meeting.item.help"
    expected_subti = 'ralph called for help in the "fancytown" meeting in ' + \
        '#fedora-meeting: "The #fedora-apps team needs people to help ' + \
        'review code http://ambre.pingoured.fr/fedora-infra/"'
    expected_secondary_icon = 'https://seccdn.libravatar.org/avatar/' + \
        '9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c' + \
        '?s=64&d=retro'
    expected_usernames = set(['ralph', 'lmacken'])
    expected_objects = set([
        'attendees/ralph',
        'attendees/lmacken',
        'channels/#fedora-meeting',
        'titles/fancytown',
    ])

    msg = {
        "username": "threebean",
        "timestamp": 1426553001,
        "msg_id": "2015-ab5c6d46-4010-4966-9d0e-dd5f7ae819a5",
        "topic": "org.fedoraproject.prod.meetbot.meeting.item.help",
        "msg": {
            "meeting_topic": "fancytown",
            "attendees": {
                "lmacken": 2,
                "threebean": 2,
                "zodbot": 2
            },
            "chairs": {
                "threebean": True,
                "lmacken": True,
            },
            "topic": "",
            "url": "threebean/2015/threebean.2015-03-17-00.42",
            "details": {
                "linenum": 4,
                "line": "The #fedora-apps team needs people to help "
                "review code http://ambre.pingoured.fr/fedora-infra/",
                "time_": 1426553001.0,
                "nick": "threebean",
            },
            "owner": "lmacken",
            "channel": "#fedora-meeting"
        }
    }


class TestSupybotMeetingItemLink(Base):
    """ As IRC meetings chug along, people can declare items for the record.
    Here's an example of someone **linking to information** on some item:
    """
    expected_title = "meetbot.meeting.item.link"
    expected_subti = (
        'mizmo linked to more information in the "Server SIG Weekly Meeting '
        '(2015-06-23)" meeting in #fedora-meeting-1: "https://bugzilla.redhat.'
        'com/show_bug.cgi?id=1234653 basically stops all media booting in most'
        ' cases"')

    expected_secondary_icon = 'https://seccdn.libravatar.org/avatar/' + \
        '2b820632db285cd534651e5f0da285d7c0a0a1bf600da16e19063163ce21db31' + \
        '?s=64&d=retro'
    expected_link = "https://bugzilla.redhat.com/show_bug.cgi?id=1234653"
    expected_usernames = set(['mizmo', 'nirik'])
    expected_objects = set([
        'attendees/mizmo',
        'attendees/nirik',
        'channels/#fedora-meeting-1',
        'titles/Server SIG Weekly Meeting (2015-06-23)',
        'topics/Open Floor',
    ])

    msg = {
        "i": 999,
        "msg": {
            "attendees": {
                "mizmo": 0,
                "nirik": 9,
                "zodbot": 5
            },
            "chairs": {
                "mizmo": True,
                "nirik": True,
            },
            "channel": "#fedora-meeting-1",
            "details": {
                "line": "https://bugzilla.redhat.com/show_bug.cgi?id=1234653 "
                "basically stops all media booting in most cases",
                "linenum": 122,
                "nick": "mizmo",
                "time_": 1435074475.0
            },
            "meeting_topic": "Server SIG Weekly Meeting (2015-06-23)",
            "owner": "nirik",
            "topic": "Open Floor",
            "url": "http://meetbot.fedoraproject.org/fedora-meeting-1/"
            "2015-06-23/fedora-meeting-1.2015-06-23-15.03"
        },
        "msg_id": "2015-2fc83ae5-2cbc-445c-85c3-266debd8df2d",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp": 1435074475.0,
        "topic": "org.fedoraproject.prod.meetbot.meeting.item.link"
    }


class TestSupybotKarma(Base):
    """ In IRC, users can give each other "karma" by saying ``user++``
    (and zodbot publishes messages for that too).

    Here's an example of that:
    """
    expected_title = 'irc.karma'
    expected_subti = 'ralph gave lmacken(36) a karma cookie in ' + \
        '#fedora-apps.  "thanks for all the fish, lmacken++"'
    expected_icon = "https://seccdn.libravatar.org/avatar/" + \
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c" + \
        "?s=64&d=retro"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "203f6cb95b44b5d38aa21425b066dd522d3e19d8919cf4b339f29e0ea7f03e9b" + \
        "?s=64&d=retro"
    expected_link = 'https://badges.fedoraproject.org/user/lmacken'
    expected_usernames = set(['ralph', 'lmacken'])
    expected_objects = set([
        'karma/lmacken',
    ])

    msg = {
        "i": 16,
        "msg": {
            'agent': 'ralph',
            'recipient': 'lmacken',
            'total': 36,
            'vote': 1,
            'channel': '#fedora-apps',
            'line': 'thanks for all the fish, lmacken++',
        },
        "topic": "org.fedoraproject.prod.irc.karma",
        "timestamp": 1345572862.556145
    }


if not 'FEDMSG_META_NO_NETWORK' in os.environ:
    TestSupybotEndMeeting.expected_long_form = full_irc_logs


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



""" Code to generate doc/topics.rst during 'sphinx-build'.

This code:

- Uses :mod:`nose` to find all the fedmsg.meta unittests.
- Extracts all the metadata and docstrings from those tests.
- Uses all that to generate a giant .rst document of all the fedmsg
  topics and what they are about with example messages.

"""

import nose
import pprint
import textwrap
import uuid

import mako.template
import six

from fedmsg.tests.test_meta import Unspecified

header = """
List of Message Topics
======================

.. DO NOT EDIT THIS DOCUMENT.

.. It is autogenerated from fedmsg_meta_fedora_infrastructure/doc_utilities.py

This document lists all the topics coming out the Fedora
Infrastructure fedmsg bus.  Example messages are included
as well as descriptions and sample output from ``fedmsg.meta``.

.. note:: All topics from Fedora Infrastructure are prefixed with
   ``org.fedoraproject.prod.``, but the :term:`topic_prefix` is omitted here
   for brevity.  For instance, the item listed as ``git.branch`` will
   actually be broadcast as ``org.fedoraproject.prod.git.branch``.

.. note:: Message bodies can contain some useful information, but be wary.
   We have done as good a job as we can *securing* fedmsg, but it is still
   a new system.  If you receive a message from pkgdb claiming that "ralph"
   is the new owner of the kernel, you should still *check* with the *actual*
   pkgdb service that this is the case.  Write code against fedmsg messages
   as a tip, but always check the authoritative source before taking any
   programmatic action.

"""

metadata_template = mako.template.Template("""
The example message above, when passed to various routines in the
:mod:`fedmsg.meta` module, will produce the following outputs:

+-----------------------------------------+${'-' * (longest + 2)}+
| :func:`fedmsg.meta.msg2title`           | ${str(title).ljust(longest)} |
+-----------------------------------------+${'-' * (longest + 2)}+
| :func:`fedmsg.meta.msg2subtitle`        | ${str(subtitle).ljust(longest)} |
+-----------------------------------------+${'-' * (longest + 2)}+
| :func:`fedmsg.meta.msg2link`            | ${str(link).ljust(longest)} |
+-----------------------------------------+${'-' * (longest + 2)}+
| :func:`fedmsg.meta.msg2agent`           | ${str(agent).ljust(longest)} |
+-----------------------------------------+${'-' * (longest + 2)}+
| :func:`fedmsg.meta.msg2usernames`       | ${repr(usernames).ljust(longest)} |
+-----------------------------------------+${'-' * (longest + 2)}+
| :func:`fedmsg.meta.msg2packages`        | ${repr(packages).ljust(longest)} |
+-----------------------------------------+${'-' * (longest + 2)}+
| :func:`fedmsg.meta.msg2objects`         | ${repr(objects).ljust(longest)} |
+-----------------------------------------+${'-' * (longest + 2)}+
| :func:`fedmsg.meta.msg2icon`            | ${str(icon_inline).ljust(longest)} |
+-----------------------------------------+${'-' * (longest + 2)}+
| :func:`fedmsg.meta.msg2secondary_icon`  | ${str(secondary_icon_inline).ljust(longest)} |
+-----------------------------------------+${'-' * (longest + 2)}+

% if not icon is Unspecified:
.. ${icon_inline} image:: ${icon}
   :height: 32px
   :width: 32px
% endif
% if not secondary_icon is Unspecified:
.. ${secondary_icon_inline} image:: ${secondary_icon}
   :height: 32px
   :width: 32px
% endif

""")

outfile = None


def write(fname, s=''):
    global outfile
    if not outfile:
        outfile = open(fname, 'w')

    outfile.write(s + '\n')


def datagrepper_link(topic):
    suffix = '.'.join(topic.split('.')[3:])
    category = topic.split('.')[3]
    base_url = 'https://apps.fedoraproject.org/datagrepper/raw'
    topic_link = base_url + '?topic=%s' % topic
    category_link = base_url + '?category=%s' % category
    tmpl = (
        "You can view the history of `messages with the %s topic <%s>`_ "
        "or `all %s messages <%s>`_ in datagrepper."
    )
    return tmpl % (suffix, topic_link, category, category_link)


def load_classes(module):
    return list(nose.loader.defaultTestLoader().loadTestsFromModule(module))


def make_topics_doc(output_dir):

    fname = output_dir + "/topics.rst"

    global outfile
    import fedmsg_meta_fedora_infrastructure.tests as source_module
    test_classes = load_classes(source_module)

    # TODO -- get the logger and announce messages
    #import fedmsg
    #filename = fedmsg.__file__
    #folder = os.path.sep + os.path.join(*filename.split('/')[:-1])
    #test_classes = load_classes(folder)

    # Strip out the conglomerator tests which are more complicated.
    test_classes = [cls for cls in test_classes if hasattr(cls.context, 'msg')]

    write(fname, header)

    for cls in test_classes:
        if not cls.context.msg is Unspecified:
            # Adjust {stg,dev} to prod.
            cls.context.msg['topic'] = cls.context.msg['topic']\
                .replace('.stg.', '.prod.')\
                .replace('.dev.', '.prod.')
            cls.context.expected_title = cls.context.expected_title\
                .replace('.stg.', '.prod.')\
                .replace('.dev.', '.prod.')

            topic = '.'.join(cls.context.msg['topic'].split('.')[3:])
            cls.__topic = topic
        else:
            cls.__topic = None

    comparator = lambda a, b: cmp(a.__topic, b.__topic)
    test_classes = sorted(test_classes, comparator)

    seen = []
    for cls in test_classes:
        if not cls.context.msg is Unspecified:
            topic = cls.__topic

            # Ignore tests that check old messages.
            if 'Legacy' in cls.context.__name__:
                continue

            # You can also exclude a test from the docs with nodoc = True
            if getattr(cls, 'nodoc', False) is True:
                continue

            modname = topic.split('.')[0]
            if not modname in seen:
                seen.append(modname)
                write(fname, modname)
                write(fname, "-" * len(modname))
                write(fname)

            write(fname, topic)
            write(fname, "~" * len(topic))
            write(fname)

            # I would use __doc__ here, but something that nose is doing is
            # stripping the __doc__ from my original unit tests.  Instead,
            # we'll use our own 'doc' attribute which is a little clumsy.
            if getattr(cls.context, 'doc', None):
                write(fname, textwrap.dedent("    " + cls.context.doc.strip()))
                write(fname)

            write(fname, datagrepper_link(cls.context.msg['topic']))
            write(fname)

            write(fname, ".. code-block:: python")
            write(fname, '\n    ' + pprint.pformat(cls.context.msg, indent=2)
                  .replace('\n', '\n    '))
            write(fname)

            # This is a unique id per entry so we don't collide image tags
            uid = str(uuid.uuid4())
            icon_inline = Unspecified
            secondary_icon_inline = Unspecified
            if not cls.context.expected_icon is Unspecified:
                icon_inline = "|%s-icon|" % uid
            if not cls.context.expected_secondary_icon is Unspecified:
                secondary_icon_inline = "|%s-secondary_icon|" % uid

            # A bunch of data for the template.
            kwargs = dict(
                link=cls.context.expected_link,
                title=cls.context.expected_title,
                subtitle=cls.context.expected_subti,
                usernames=cls.context.expected_usernames,
                agent=cls.context.expected_agent,
                packages=cls.context.expected_packages,
                objects=cls.context.expected_objects,
                icon_inline=icon_inline,
                secondary_icon_inline=secondary_icon_inline,
            )

            def length(value):
                """ Find longest string so we can pad our tables adequately """
                if isinstance(value, six.string_types):
                    return len(value)
                return len(repr(value))
            longest = max([length(value) for value in kwargs.values()])

            write(fname, metadata_template.render(
                icon=cls.context.expected_icon,
                secondary_icon=cls.context.expected_secondary_icon,
                Unspecified=Unspecified,
                longest=longest,
                **kwargs
            ))
            write(fname)

    outfile.close()

if __name__ == '__main__':
    make_topics_doc()



# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors: Michael Simacek <msimacek@redhat.com>
#
""" Tests for Koschei messages """

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestKoscheiPackageStateChange(Base):
    """ Koschei is a continuous integration system for RPM packages. It tracks
        dependency changes done in Koji repositories and rebuilds packages whose
        build dependencies change too much.

    This message type is published by Koschei when package's build or resolution
    state changes.
    """

    expected_title = "koschei.package.state.change"
    expected_subti = "rnv's builds started to fail in Fedora 22 (arm)"
    expected_link = 'https://apps.fedoraproject.org/koschei/package/rnv'
    expected_secondary_icon = \
        'https://apps.fedoraproject.org/packages/images/icons/rnv.png'
    expected_packages = set(['rnv'])

    msg = {
        "username": "msimacek",
        "i": 2,
        "timestamp": 1412260063,
        "msg_id": "2014-45d99116-f93e-45ef-8611-04fa2eabbb82",
        "topic": "org.fedoraproject.dev.koschei.package.state.change",
        "msg": {
            "name": "rnv",
            "old": "ok",
            "new": "failing",
            "koji_instance": "arm",
            "repo": "f22",
            "collection": "f22",
            "collection_name": "Fedora 22",
            "groups": ["c", "xml"]
        }
    }


add_doc(locals())



# This file is part of fedmsg.
# Copyright (C) 2013 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for compose messages """

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestPungiKojiStart(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when a compose starts.
    """
    expected_title = "pungi.compose.status.change"
    expected_subti = "pungi-koji compose of Fedora-24-20151208.n.7 started"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_objects = set(['rawhide/Fedora-24-20151208/n/7'])
    msg = {
        "msg": {
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
            "status": "STARTED"
        },
        "timestamp": 1449600728.0,
        "topic": "org.fedoraproject.prod.pungi.compose.status.change"
    }


class TestPungiKojiComplete(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when a compose starts.
    """
    expected_title = "pungi.compose.status.change"
    expected_subti = "pungi-koji compose of Fedora-24-20151208.n.7 just finished"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_objects = set(['rawhide/Fedora-24-20151208/n/7'])
    msg = {
        "msg": {
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
            "status": "FINISHED"
        },
        "timestamp": 1449600728.0,
        "topic": "org.fedoraproject.prod.pungi.compose.status.change"
    }


class TestPungiPhaseStart(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when the ``createrepo`` phase of a
    compose starts.
    """
    expected_title = "pungi.compose.phase.start"
    expected_subti = "pungi-koji started the createrepo phase of the " + \
        "Fedora-24-20151208.n.7 compose"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_objects = set(['rawhide/Fedora-24-20151208/n/7'])
    msg = {
        "i": 1,
        "timestamp": 1449605930.0,
        "msg_id": "2015-05ea75ce-f648-41b5-9e29-da56202a2ba5",
        "topic": "org.fedoraproject.prod.pungi.compose.phase.start",
        "msg": {
            "phase_name": "createrepo",
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
        }
    }


class TestPungiCreateISOTargets(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when createiso targets are assigned.
    """
    expected_title = "pungi.compose.createiso.targets"
    expected_subti = "pungi-koji assigned 3 createiso targets"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_objects = set([
        "mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/"
        "armhfp/iso/Fedora-24-20151208.n.7-Server-armhfp-dvd1.iso",
        "mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/"
        "i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso",
        "mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/"
        "x86_64/iso/Fedora-24-20151208.n.7-Server-x86_64-dvd1.iso",
    ])
    msg = {
        "i": 1,
        "timestamp": 1449605930.0,
        "msg_id": "2015-05ea75ce-f648-41b5-9e29-da56202a2ba5",
        "topic": "org.fedoraproject.prod.pungi.compose.createiso.targets",
        "msg": {
            "deliverables": [
                "/mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/armhfp/iso/Fedora-24-20151208.n.7-Server-armhfp-dvd1.iso",
                "/mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso",
                "/mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/x86_64/iso/Fedora-24-20151208.n.7-Server-x86_64-dvd1.iso"
            ],
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
        }
    }


class TestPungiCreateISOWin(Base):
    """ In 2016, the `release engineering
    <https://fedoraproject.org/wiki/ReleaseEngineering>`_ team started moving
    the compose process to ``pungi4``, which emits messages like this as it
    does its composition work.

    Here's an example message published when a createiso target completes.
    """
    expected_title = "pungi.compose.createiso.imagedone"
    expected_subti = "pungi-koji finished createiso for " + \
            "Server/i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso"
    expected_link = "http://kojipkgs.fedoraproject.org/" + \
        "compose//rawhide/Fedora-24-20151208.n.7"
    expected_objects = set([
        "mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose/Server/"
        "i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso",
    ])
    msg = {
        "i": 1,
        "timestamp": 1449605930.0,
        "msg_id": "2015-05ea75ce-f648-41b5-9e29-da56202a2ba5",
        "topic": "org.fedoraproject.prod.pungi.compose.createiso.imagedone",
        "msg": {
            "variant": "Server",
            "arch": "i386",
            "file": "/mnt/koji/compose/rawhide/Fedora-24-20151208.n.7/compose"
            "/Server/i386/iso/Fedora-24-20151208.n.7-Server-i386-dvd1.iso",
            "compose_id": "Fedora-24-20151208.n.7",
            "location": "http://kojipkgs.fedoraproject.org/compose/"
            "/rawhide/Fedora-24-20151208.n.7/compose",
        }
    }



add_doc(locals())
